<?php
 /** 
 * $Id: carbon_calculate.inc,v 1.6 2006/12/20 13:36:50 johnackers Exp $ 
 * 
 * Drupal carbon module (calculations) 
 *
 * Created on 18-Oct-2006
 * 
 * @file 
 * The calculations used to build carbon footprint.
 * 
 * Different carbon_stamps have a different scope, some declare 
 * emissions on one date, others declare emissions over a period
 * of time. 
 *    
 * Some sources of emissions e.g. from air travel need special 
 * formulas for calculating resulting CO2 because fuel used in 
 * aircraft takeoff has to be taken into consideration. 
 */

/** 
 * Create a printable carbon footprint for a given period (usually a year) 
 * based on on all the available carbon_stamps.  
 * 
 * @param $ar array of all carbon_stamps (with associated carbon_source fields)
 * @param $start first date the generated carbon footprint 
 * @param $end last date of the generated carbon footprint
 *  
 * @return a string containing a table that shows the carbon footprint.
 */

 
function carbon_calculate(&$ar, $start, $end)
{
  $header = array("title","source","scope","num","adjust","convert","Kg of CO2" );
  $rows = array();    
  $dgroup = array(); 
  $grand_total = 0 ; 
  foreach ($ar as $node)
  {
    $dgroup[] = _concat_same_value($node);
    $node->total = 234 ; 
  }

  // each key represents a group of readings
  // that can be considered identical 
  
  $keys = array_unique($dgroup);
  $numupdates = 0 ;    
  $notes = array();              
  foreach ($keys as $subgroup)
  {
  	// pull out the first group and 
    // select all the nodes that match that group

    $select = array(); 
    $column_notes = "" ;   
    foreach ($ar as $node2)
    {
      if (_concat_same_value($node2) != $subgroup) 
        continue ; 
      $select[] = $node2;  
      $node2 ->total = null ;
    }

    if (!empty($result->notes))
    {
      $notes[] = $result->notes; 
      $column_notes =  " [".count($notes)."]" ;  
    }
    
    $fname = '_scope_'.$select[0]->scope; 
    $result_as_text = null ; 
    if (function_exists($fname))
    {
      $result = ($fname($select, $start, $end));
       
      if (!empty($result->error))
      {
        $result_as_text	= $result->error ; 
      }
      else
      {
        if (!empty($result->subtotal))  
          $result_as_text = sprintf("%.0f",$result->subtotal);
      }
    }
    else
        $result_as_text = $fname." does not exist, invalid scope ";
        
    if (!empty($result_as_text))
    {
      $rows[] = array(carbon_link_from_title($select[0]),
        $select[0]->sourcetitle, 
        _get_scope_short_description($select[0]->scope),
        count($select). $column_notes, 
        $select[0]->adjustment, 
        $select[0]->multiplier, 
        $result_as_text);
      $node2 ->total =  $result->subtotal ;
      $grand_total += $result->subtotal ; 
      $numupdates++ ;
    } 

  }
  if (!empty($numupdates))
  {
    $out = "<h2 class='title'>Emisssions from ".format_date($start, "large")."</h2>" ;
    $rows[] = array("total", sprintf("%.3f metric tonnes",$grand_total/1000),null, null, null, null, sprintf("%.0f",$grand_total)); 
    $out .= theme('table', $header, $rows);

    $i = 1 ; foreach ($notes as $note)
    {
    	 $out .= "<p class='link'>[".$i++."] ".$note."</p>" ; 
    }
  }
  return $out ; 
}

/**
 * identify stamps that are identical so that they can be
 * group together. 
 * 
 * meter readings (scope == 1) are handled as a group, so stamps 
 * with a different value should be group together. 
 * 
 */


function _concat_same_value($node){
  $hashcode = $node->title.'.'.$node->code.'.'.$node->scope.'.'.$node->adjustment.'.'.$node->sourceclass;
  return $node->scope == 1 ? $hashcode : $hashcode.'.'.$node->reading ; 
}

/**
 * identify stamps that are identical that can presented in a group on a form
 * and the only the field that needs to be changed is enddate and reading.
 */

function _concat($node){
  return $node->title.'.'.$node->code.'.'.$node->scope.'.'.$node->adjustment.'.'.$node->sourceclass; 
}


/**
 * todo check that no ; or : present
 */

function _evaluate($expression)
{
  $expression = trim($expression); 
  $exp1 = substr($expression,0,1); 
  if (strcmp($exp1,"=")==0)
    $expression = substr($expression,1);

  return eval('return '.$expression.';'); 
}


class Result 
{

}    



function _sort_by_enddate($ar)
{
  $ar1 = array();
  foreach ($ar as $node)
  {
    $ar1[$node->enddate] = $node ;      
  }       
  ksort($ar1);
  return array_values($ar1);
}


/**
 * There are two scenarios.  
 * 1. the emissions occur on a single date (e.g. air travel) 
 * 2. the emissions are spread over a year. 
 */

function _scope_0(&$ar, $start, $end)
{    
  $result = new Result(); 
  foreach ($ar as $node)  
  {  	
    if (empty($node->startdate))
    {
        // this is the simple case where all the emissions 
        // were released  on the same date 
        // (take care with < and <= etc to include all stamps in 
        // at least one accounting period (a year)
        //
         
      if ($node->enddate < $start) continue ; 
      if ($node->enddate >= $end) continue ;
      $result->subtotal += _invoke_calculate($node, &$result->notes);
    }
    else
    {
        // this is the case where the emissions spread across a long 
        // period of time e.g. the carbon stamp might refer to emissions
        // from june 05 to june 06 and so would appear in two separate years. 
         
      if ($node->enddate < $start) continue ; 
      if ($node->startdate >= $end) continue ; 
                	
      $adjust = 1.0 ; 
      if ($node->startdate < $start)
      { 
        $adjust *=  ($node->enddate - $start)/($node->enddate - $node->startdate);  
      }
      if ($node->enddate >= $end)
      { 
        $adjust *=  ($end - $node->startdate)/($node->enddate - $node->startdate);  
      }
      $result->subtotal += _invoke_calculate($node, &$result->notes) * $adjust ;
//      drupal_set_message("using adjust :". $adjust, "info");
    }
    return $result ; 
  }
}

/**
 * meter readings 
 */


function _scope_1(&$ar1, $start, $end)
{ 
  $result = new Result(); 
  $result->error = array();
    
  $num = sizeof($ar1);

  if ($num == 0)
  {
    $result->error = "No readings" ; 
    return $result ; 
  }

//  drupal_set_message("array has1 ".$num, "info"); 
  $time_sorted_map = _sort_by_enddate($ar1);
//  drupal_set_message("array has2 ".$num, "info");

  // do not attampt to make sense of requests for for 
  // consumption that fall entirely below or entirely above 
  // the range of available meter readings 
  
  $time_sorted_ar = array_values($time_sorted_map); 
   
  if ($end < $time_sorted_ar[0]->enddate)
    return $result ; 
    
  if ($start > $time_sorted_ar[count($time_sorted_ar)-1]->enddate)
    return $result ; 

  $result->start = _find_closest($time_sorted_map, $start);            
  $result->end = _find_closest($time_sorted_map, $end);    


  if ($result->start == $result->end)
  {
    $result->error = "Can only find one suitable meter reading" ;
    return $result ; 
  }

  $consumption = $result->end->reading - $result->start->reading ; 
  if ($consumption < 0)  
  {       
    $result->error = "Meter appears to have gone backwards ".$consumption ;
    return $result ; 
  }               
  $period = $result->end->enddate - $result->start->enddate ; 

  if ($period == 0)  
  {       
    $result->error = "No suitable meter readings" ;
    return $result ; 
  }
  
  if ($period < 0)  
  {       
    $result->error = "Time appears to have gone backwards" ;
    return $result ; 
  }
  // not handling leap years properly
  $result->subtotal = $consumption * (365*24*60*60)/$period ; 


  $result->notes = "Using readings ". 
  $result->start->reading. " on " . substr(format_date($result->start->enddate, "small"),0,10).
   " and ". 
  $result->end->reading. " on " . substr(format_date($result->end->enddate, "small"),0,10);  
  return $result ; 
}

/**
 * daily event
 * The only difference betweeen a scope_0 and scope_2 event is that 
 * the scope_2 event repeats the same level of emissions every
 * day.   
 */

function _scope_2(&$ar, $start, $end)
{ 
  // work out how many days this sequence of events occurred

  $days = ($end - $start)/(60*60*24); 
  $r2 = new Result(); 
  foreach ($ar as $node)
  {
  	$result = _scope_0($ar1 = array($node), $start, $end);
    $r2->subtotal =  $days * $result->subtotal ;  
  }
  return $r2 ; 
}


/**
 * walk through the array and look for carbon_stamps that are 
 * closest to the required date - used to find meter readings 
 * near the start and end dates of a carbon footprint. 
 *
 * @param $ar array of all carbon_stamps 
 */

function _find_closest($ar, $date)
{
  $diff_found = null  ; 
  $node_found = null ;
  foreach ($ar as $node)
  {
    $d = $node->enddate - $date ;   
    if ($d < 0) $d = -$d ; 
    if ($diff_found == null || $d < $diff_found)
    {
      $diff_found = $d ;  $node_found = $node; 
    }
  }
  return $node_found ;    
}


/**
 * Each carbon source can have a different classname.
 * Invoked from carbon_source.inc
 * 
 * @return a map of class names (keys) and descriptions (values). Descriptions (values) not used yet.
 */

function carbon_calculate_class_map()
{
  return array(
    "simple"        =>  t("simple multiplier"),
    "climatecare" =>    t("model used by commercial offsetting company"),
    "chooseclimate" =>  t("model created by climate mathamatician")
    );
}


/**
 * 
 * At the moment the classname is simply mapped to a function name.
 *
 * @param $node a carbon stamp 
 */


function _invoke_calculate($node, $notes)
{ 
  $class = '_'.$node->sourceclass ; 
  return  $class($node, $notes);
}
 
/**
 * The simple class calculates emissions with the formula 
 * 
 * emissions = r*m*a 
 * 
 * where r is the value spcified by the user 
 *       m is the multiplier used to convert units of r 
 *       a is the adjustment specified by the user
 *       which might be 48/52 to allow for holidays
 *       from a daily commute.   
 *       (this can be a string and must be evaluated)
 *
 * to the corresponding CO2 value. 
 * 
 * @param $node a carbon stamp 
 */
 
function _simple($node, &$notes)
{ 
  return  $node->reading * $node->multiplier * _evaluate($node->adjustment) ; 
}

/**
 *  This formula comes from http://www.chooseclimate.org 
 * 
 
    Fuel per passenger (Data for B-747).
    
    Fuel = [7840 + 10.1 * (distance-250)] (*2 if return)
    (7840 kg take off-climb-descent, 10.1 kg/km cruising.
    Passengers, = 370 * [occupancy] (/1.5 if business) 
    Greenhouse warming:
    CO2 = fuel * (44/12 * 156/184) (molecular masses)

    Total warming effect of CO2,Ozone (made by NOx), water vapour and 
    contrails is about three times greater than effect of CO2 alone
 */


/**
 * $multiplier should only be used to convert distance from some 
 * arbitrary unit to Km. 
 **/

function _chooseclimate($node, &$notes)
{
  $model = array(
    "aircraft" => "B737-400",
    "fuel_for_takeoff" => 7840,
    "fuel_cruising_per_km" => 10.1,    
    "climb_distance" => 250,  
    "capacity" => 370,  
    "occupancy" => .80,
    "fuel_conversion" => (44/12 * 156/184),
    "radiative_forcing_index" => 2.7
    );    
  return _flight_calculate($node, &$notes, $model);
}

/**
     Climate care model 

This model is based on the information in a repoprt written 
by the Environmnetal Change Institute for climate care 
and is provided on the climate care website. 
http://www.climatecare.org/_media/documents/pdf/Aviation_Emissions_&_Offsets.pdf

the raw data refrred to on aircraft fuel consumption
http://reports.eea.europa.eu/EMEPCORINAIR3/en/B851vs2.4.pdf

my version of this with average fuel consumption added: 
http://cvs.drupal.org/viewcvs/ checkout /drupal/contributions/modules/carbon/air_fuel_use.html

note the climate care assume 100% occupancy and that on long 
haul flights the 10% of fuel can be be attributed to freight. 
 */

function _climatecare($node, &$notes)
{
  $distance_km = $node->reading * $node->multiplier ;

  // switch planes depending on distance 

  $model =  $distance_km < 3500 ? _climatecareB737() : _climatecareB747();  
  
  return _flight_calculate($node, &$notes, $model);
}

function _climatecareB747() {     
  return array(
    "aircraft" => "B747-400",
    "fuel_for_takeoff" => 3402,
    "fuel_cruising_per_km" => 10.1,    // from average kg/mile in EU table
    "climb_distance" => 0,  
    "fuel_conversion" => 3.15,         // http://www.climatecare.org/_media/documents/pdf/Aviation_Emissions_&_Offsets.pdf
    "capacity" => 400,                 // ditto
    "occupancy" => 1.00,               // ditto
    "radiative_forcing_index" => 2.0   // ditto & http://iacweb.ethz.ch/tradeoff/
    );    
}

function _climatecareB737() {   
  return array(
    "aircraft" => "B737-400",
    "fuel_for_takeoff" => 825.4,
    "fuel_cruising_per_km" => 3.01,    // from average kg/mile above
    "climb_distance" => 0,  
    "fuel_conversion" => 3.15,         // http://www.climatecare.org/_media/documents/pdf/Aviation_Emissions_&_Offsets.pdf
    "capacity" => 180,                 // ditto
    "occupancy" => 1.00,               // ditto
    "radiative_forcing_index" => 2.0   // diito & http://iacweb.ethz.ch/tradeoff/
    );    
}

function _climatecareA340() {   
  return array(
    "aircraft" => "Airbus-A340",
    "fuel_for_takeoff" => 2019,
    "fuel_cruising_per_km" => 6.6,    // from average kg/mile in EU table
    "climb_distance" => 0,  
    "fuel_conversion" => 3.15,         // http://www.climatecare.org/_media/documents/pdf/Aviation_Emissions_&_Offsets.pdf
    "capacity" => 295,                 // ditto
    "occupancy" => 1.00,               // ditto
    "radiative_forcing_index" => 2.0   // diito & http://iacweb.ethz.ch/tradeoff/
    );    
}


/**
 * the choose climate model is basically a simple
 * first order polynomial and I have used it for 
 * the climate care model as well. 
 */


function _flight_calculate($node,  &$notes, $model)
{ 
  $distance = $node->reading * $node->multiplier ;
  
  $flight_takeoff = $model["fuel_for_takeoff"] ;  
    
  if ($distance > $model["climb_distance"]) 
    $flight_cruising += ($model["fuel_cruising_per_km"] * ($distance-$model["climb_distance"]));  // Kg
      
  $fuel = ($flight_takeoff + $flight_cruising)/($model["capacity"] * $model["occupancy"]) ;   //.Kg 
    
  $co2 = $fuel * $model["fuel_conversion"];  // convert fuel to CO2
  
  // the radiative_forcing_index below (expected to be between 
  // 1.8 and around 2.7) includes effect of the non CO2 gasses

  $other_gasses = $co2 * $model["radiative_forcing_index"] ;
  
  $notes =  sprintf("%s uses %.2d Kg of fuel on takeoff/landing, ". 
            " and %.2d Kg while cruising; ". 
            " CO2 per passenger is %.2d Kg," . 
            " all gasses (expressed as CO2 equivalent) is %.2d Kg ",  
            $model["aircraft"], $flight_takeoff,  $flight_cruising, $co2, $other_gasses); 

  return $other_gasses * _evaluate($node->adjustment) ; // Kg        
}

 
?>
