<?php
/**
 * $Id: carbon_footprint.inc,v 1.9 2006/12/20 13:35:39 johnackers Exp $ 
 * 
 * Drupal carbon footprint module 
 *
 * Created on 18-Oct-2006
 * 
 * @file 
 * definition of carbon_footprint node. Each carbon_footprint
 * node contains settings such as choice of preferred carbon 
 * model and privacy settings that control the presentation of 
 * a carbon footprint.   
 */



/**
 * Render a page listing a one line summary of all carbon footprints 
 *
 * Protection is complicated because there is support for groups 
 * and sharing. 
 * 
 * Note that the param $uid is the user we are focused on, not the user doing 
 * the browsing.
 *
 * @param $uid select by uid; set to zero to retrieve footprints from all users
 * @param $org select by organisation; set to '' or null to retrieve footprints 
 * from users in any goup
 * @param $map results returned here 
 */ 


function carbon_footprint_page(&$map, $uid, $org) {
  global $user;

  $header = array(
    array("data" => t("account name"), "field" => "title","sort" => "asc"),
    array("data" => t("postcode"), "field" => "postcode","sort" => "asc"),
    array("data" => t("group"), "field" => "organization"),
    array("data" => t("first date"), "field" => "firstdate"),
    array("data" => t("status"), "field" => "filing_status"),
    array("data" => t("#stamps"), "field" => "numstamps"),
    array("data" => t("filed by"), "field" => "name")
  );

  // content variable that will be returned for display
  $page_content = '';

  $view_all = carbon_access("view", "carbon_footprint"); 

  $sql =     "SELECT n.nid, n.title, n.uid, n.type, f.*, name, count(*) as numstamps " .
             ",s.uid AS shared_uid, protection " .
      		   "FROM {carbon_footprint} f " .
             "LEFT  JOIN {carbon_stamp_footprint} j ON  f.nid = j.fid " .  // to get num stamps
             "LEFT  JOIN {carbon_footprint_share} s ON  f.nid = s.nid " .  // to get perm to access
             "INNER JOIN {node} n ON f.nid = n.nid " .
             "INNER JOIN {users} u ON n.uid = u.uid " .
             "WHERE (u.uid = %d " .          (empty($uid) ?  " OR 1) " : ") ").     // opt select by user 		
             "AND (f.organization = '%s' " . (empty($org) ?  " OR 1) " : ") ") ;    // opt select by org

  // non admin users only permitted to see shared footprints and 
  // footprints from same group or public footprints. 
  // have to filter in the SQL in case there are a lot of records
  // Note. private footprints in same group are shown but not linked

  if (!$view_all)         
    $sql .=  "AND (  (s.uid = %d)  " .   // ie shared with target user
             "    OR (n.status AND length(f.organization) > 1 and f.organization = '%s') " .
             "    OR (n.status AND protection >= 2) " .
             "    OR (n.uid = %d) ".
             "    OR  1 " .
             "    ) " ;
  $sql .=    "GROUP BY f.nid " ; 
  $sql .= tablesort_sql($header); 

//  drupal_set_message(" my sql ". $sql . " org:". $org. " uid:" . $uid, 'info'); 
    
  // get the links (no range limit here)
  $queryResult = db_query($sql, $uid, $org, $uid, $org, $user->uid);
  $rows[] = array();
  $filestat = _filing_statuses();

  $savedrecord = null; 	
  while ($record = db_fetch_object($queryResult)) 
  { 
    $title = carbon_anonomize($record->title);   
    $access = carbon_access("view", $record) or ($org == $record->organization) ;
	  $link = $access ? l($title,"node/".$record->nid) : $title; 
 
	  $rows[] =  array($link,
				$record->postcode,
				l($record->organization,'carbon/footprint/'.$record->organization),
				empty($record->firstdate) ? '' : format_date($record->firstdate,"custom", "Y-m-d"), 
				$filestat[$record->filing_status],
				$record->numstamps == 1 ? "<= 1" : $record->numstamps,				
				l($record->name,"user/".$uid));
    
    // we may not be displaying the footprints but simply building 
    // a map of footprints that we could display if we were asked. 
                
    if (isset($map) && $access)
    {
        $map[$record->nid] = $record ; 
    }
    $savedrecord = $record ; 
    
  }

  if (!empty($org)) 
    $title = t("Carbon Footprints for '%name'", array('%name' => $org));
 
  elseif (!empty($uid)) 
    $title = t("Carbon Footprints owned by '%name'", array('%name' => $savedrecord->name));
    
  else 
    $title = t("Carbon Footprints");
   
  drupal_set_title(check_plain($title));
       
  $output = theme("table", $header, $rows);
  $output .= theme("pager", NULL, 50, 0);

  $links = array();
  if (isset($savedrecord))
  {
    if (isset($uid))
      $links = carbon_link("node", $savedrecord); 
      
    elseif (isset($grp))
    {
      $record->organization = $grp ; 
      $links = carbon_link("node", $savedrecord);
    }     
    if (count($links))
      $output .= "<div class='links'>";   

    foreach ($links as $link)
    {
      $output .= $link . " | " ;
    }
    if (count($links))
      $output .= "</div>" ;   
  }
  return $output;
}


function carbon_page_links($node) 
{
    $links = carbon_link(); 
}



/**
 * Implementation of hook_access().
 * 
 * Refer to carbon_access because it is easier to check 
 * access for all carbon types in one place.  
 */


function carbon_footprint_access($op, $node)
{
	return carbon_access($op, $node);
}


/**
 * Return an array of protection options
 * @return array of years
 */
function _get_protection_options() {
  return array(t("private"), t("share with others in my group"), t("share with everybody"));
}


/**
 * Implementation of hook_form().
 */


function carbon_footprint_form(&$node, &$param) {
  global $user;
      
  $source = carbon_source_fetch_array();    
    
  $model_map = _get_model_map($source);
  $source_map = _get_source_map($source);
  $source_keys = array_keys($source_map); 

  $form = array();  
  $form = carbon_footprint_account_form($form, $node, $model_map); 
  
  // if this is a new node, don't accept stamps yet

//  if (!isset($node->nid) || $node->nid == -1)
//    return $form ;    

  $i = $group = 0 ;   
    
  $processed = array() ;  

  
  if (!isset($node->stamp))
    $node->stamp = array(); 
    
  foreach ($node->stamp as $stamp ) 
  {                
    foreach($source_keys as $key)
    {     
        // we have identified on of the utilities
        // so now go through the list again
            
      $grouphash = _concat($stamp);
             
      if (!array_key_exists($grouphash, $processed))
      { 
      // we have identified a group created by the user 
      // that has not been displayed so create a set 
      // of forms for EACH source in the group/
            
        $add_new_field = false; 
        foreach ($node->stamp as $stamp3)
        { 
          if ($grouphash == _concat($stamp3))
          {                             
             _stamp_repeating_form($form, $stamp3, $stamp3, $i, $group); 
             $i++  ;                             
             $add_new_field = true ; 
          }                                       
        }
        if ($add_new_field)
        {
          _stamp_repeating_form($form, null, $stamp, $i, $group); 
          $i++ ;
        }
        $group++ ;
        $processed[$grouphash] = 0 ;
      }
    }             
  }
  $existing_codes = array();
  foreach ($node->stamp as $stamp4)
    $existing_codes[] = $stamp4->code ;


  $forms = carbon_footprint_template(); 
  foreach ($forms as $template)
  {
    if (_search_preg($template["code"], $existing_codes))
      continue ;    // we already got some of these so dont prompt
      
    // check that code not already being used, in which case 
    // the user has picked up the ball and is running 
    
    _stamp_form($form, $source_keys, array_values($source_map), null, $i, $group, $template["title"]);    
    _stamp_customize_form($form, $source_keys, array_values($source_map), null, $i, $group, $template); 
    $i++ ; 
    $group++ ;
  }
  // for the benefit of advanced users
  _stamp_form($form, $source_keys, array_values($source_map), null, $i, $group, "My other source of emissions");
  $stamp_form = "form.".$i ; 
  $stamp_group_form = "form.".$group ;
  $form[$stamp_group_form]["#collapsed"] = true ; 
    
  $i++ ; $group++ ;
  return $form ;  
} 

/**
 * search an array of strings $target with 
 * another array of patterns and look for 
 * a match. 
 * 
 * @return true if match
 */
  
function _search_preg($pattern, $subject)
{
	foreach ($subject as $t)
  {
    foreach($pattern as $p)
    {
    	if (preg_match("/".$p."/", $t))
        return true ; 
    }
  }  
  return false;     
}



/**
 * This template defines the default categories that users
 * might want to use to categorise their different sources 
 * of emissions.
 */

function carbon_footprint_template()
{
  return array(
    array(
        "title" => "My home gas",         
        "scope" => array(0), 
//      "weight" => 1,
        "code" => array("gas-m3", "gas-.*"), 
        "description" => "amount of gas used in the last year in cubic metres."),
        
    array(
        "title" => "My home electricity", 
        "scope" => array(0), 
//      "weight" => 2,
        "code" => array("elec-kwh","elec-.*"),
        "description" => "amount of electricity used in the last year in Kilowatt Hours."),   
         
    array(
        "title" => "My commute to work", 
        "scope" => array(2), 
//      "weight" => 3,   
        "code" => array("car-med-miles", "car-.*", "pt-.*"),
        "description" => "enter the distance you travel to work each day in miles."),
        
    array("title" => "My air miles", 
        "scope" => array(0),
//      "weight" => 4,   
        "code" => array("air-miles", "air-.*"),
        "description" => "enter your total air miles in the last year.")
    );  
}

/**
 * So as not to horify initial users with a lot of fields we present 
 * a small number of typical 
 */

function _stamp_customize_form(&$form, $source_keys, $source_descriptions, $stamp, $i, $group, $template) 
{
  $stamp_form = "form.".$i ; 
  $stamp_group_form = "form.".$group ;
   
  $code_index = array_search($template["code"][0],$source_keys);
  
  $form[$stamp_group_form][$stamp_form]["form.".$i.".title"]["#default_value"]= $template["title"] ;
    
  $form[$stamp_group_form][$stamp_form]["form.".$i.".code_index"]["#type"] = "hidden" ;
  $form[$stamp_group_form][$stamp_form]["form.".$i.".code_index"]["#value"] = $code_index ; 
  
  $form[$stamp_group_form][$stamp_form]["form.".$i.".scope_index"]["#type"] = "hidden" ;
  $form[$stamp_group_form][$stamp_form]["form.".$i.".scope_index"]["#value"] = $template["scope"][0]; 

  unset($form[$stamp_group_form][$stamp_form]["form.".$i.".title"]["#description"]);    
  unset($form[$stamp_group_form][$stamp_form]["form.".$i.".startdate_s"]["#type"]);
  unset($form[$stamp_group_form][$stamp_form]["form.".$i.".enddate_s"]["#type"]);  
  unset($form[$stamp_group_form][$stamp_form]["form.".$i.".adjustment"]["#type"]);
    
  $form[$stamp_group_form][$stamp_form]["form.".$i.".reading"]["#description"] = $template["description"];

  $form[$stamp_group_form]["#collapsed"] = true ; 
  $form[$stamp_group_form]["#weight"] = $template["weight"]  ; 
} 



function carbon_footprint_edit($nid)
{
    $empty = array(); 
    $node = new Node();
    $node->nid = $nid ; 
    $node = node_load($nid);	
    $form = carbon_footprint_form($node, $empty);
    
    $output .= drupal_get_form('carbon_stamp_table_edit', $form);
    
    return $output ; 
}


/**
 * Return an array of task filing_statuses
 * @return array of filing_statuses
 */
function _filing_statuses() {
  return array("draft", "filed", "audited");
}


function carbon_footprint_account_form(&$form, $node, $model_map)
{	
  $form["account"] = array(
			"#type" => "fieldset", 
			"#title" => "My preferences",
    		"#weight" => -1,
//      '#prefix' => '<div class="container-inline">',
//      '#suffix' => '</div>',  
			"#collapsible" => TRUE,
			"#collapsed" => TRUE);

  $form["account"]["title"] = array(
	  "#type" => "textfield", 
    "#title" => t("Title of this footprint"), 
    "#required" => TRUE, 
    "#default_value" => empty($node->title) ? $node->name : $node->title,
    "#size" => 20,
    "#maxlength" => 40, 
    "#weight" => -1,
    "#description" => t("Usually your name or someone else's name and can be different from your username."));
    
  if (variable_get("carbon_use_postal_code", 1))
  
    $form["account"]["postcode"] = array(
	  "#type" => "textfield", 
    "#title" => t("Postcode"), 
    "#required" => FALSE, 
    "#default_value" => empty($node->postcode) ? '' : $node->postcode,  
    "#size" => 10,
    "#maxlength" => 20, 
    "#weight" => -1,
    "#description" => t("Either some or all of your postcode or zipcode. Interesting to others " .
            "if you share your footprint."));
            
  if (variable_get("carbon_use_organization", 1))
  
    $form["account"]["organization"] = array(
    "#type" => "textfield",
	  "#title" => t("Local Group"),
	  "#default_value" => empty($node->organization) ? '' : $node->organization,
    "#size" => 20,
    "#maxlength" => 20, 
    "#description" => t("Name of any local group, leave blank if none."),
    "#required" => FALSE);

  $model_keys = array_keys($model_map); 
  if (variable_get("carbon_use_model_selection", 1))
  
    $form["account"]["model_index"] = array(
    "#type" => "select", 
    "#title" => t("Preferred emissions model"), 
    "#required" => FALSE, 
    "#options" => array_values($model_map), 
    "#default_value" => isset($node->model) ? array_search($node->model, $model_keys) : 0 ,
    "#description" => "Select the preferred model that you want to use to calculate the CO2 emissions." .
        		" Not all the models support all possible <a href='/?q=carbon/source'>carbon " .
        		" sources</a>; in which case substitutes will be made. Leave as -no preference- if you are not sure.",
    "#weight" => 2);

  $form["account"]["protection"] = array(
	  "#type" => "select", 
	  "#title" => t("Privacy"), 
	  "#required" => FALSE, 
	  "#default_value" => !isset($node->protection) ? 2 : $node->protection,
    "#description" => t("Who can see this carbon footprint?"),
	  "#options" => _get_protection_options(),
	  "#weight" => 2);


  $form["account"]["firstdate_s"] = array(
		"#type" => "textfield", 
    "#title" => t("starting date for emission reports"),  
	  "#required" => FALSE, 
    "#size" => 20,
    "#maxlength" => 20, 
	  "#default_value" => empty($node->firstdate)  ? "" : format_date($node->firstdate, "custom", "Y-m-d H:i:s O"),
	  "#weight" => 3,
	  "#description" => 
                t("Format: %time. Can be left blank. It will default to Jan 1st of the first year " .
                "for which any data has been entered.", 
                array("%time" => format_date(time(), "custom", "Y-m-d"))));
  return $form;
}



/**
 * Validate our forms
 * @param string form id
 * @param array form values
 */
function carbon_footprint_validate($form_id, $form_values) 
{
  carbon_stamp_validate($form_id, $form_values); 
}




/**
 * Implementation of hook_view, add our node specific information
 * @param node object to display
 * @param boolean is this a teaser or full node?
 * @param boolean is this displaying on its own page
 */
function carbon_footprint_view(&$node, $teaser = FALSE, $page = FALSE) 
{
//	if (!user_access('view carbon footprints')) {
//		return true;
//	}

  $node = node_prepare($node, $teaser);
  $carbon_footprint_info = theme("carbon_footprint_table_view", $node);  // try item_list
  $node->body .= $carbon_footprint_info;
  $node->teaser .= $carbon_footprint_info;
}



/**
 * Theme function to display additional node data
 * @param node to display
 * @return HTML string with additional node information
 */
 
 
// then create an array of column names


/**
 * Theme function to display additional node data
 * @param node to display
 * @return HTML string with additional node information
 */
 
 
// then create an array of column names

function theme_carbon_footprint_table_view($node)
{
	$header = array(
	  array("data" => t("date"), "field" => "enddate"),
	  array("data" => t("title"), "field" => "sourcetitle"),
    array("data" => t("model"), "field" => "sourcemodel"),
	  array("data" => t("emission"), "field" => "title"),
	  array("data" => t("reading"), "field" => "reading"),
	  array("data" => t("units"), "field" => "units"),
	  array("data" => t("adjust"), "field" => "adjustment"),
	//    array("data" => t("type or size")),
	);
	

  	// content variable that will be returned for display
	$page_content = '';
	$grand_total = 0.0 ;

	$ar = array();	 

	$firstdate = time() ;  $lastdate = 0 ;  // default first report is today
  if (count($node->stamp) == 0)
    return("");
    
  foreach ($node->stamp as $stamp)        // now find out where the stamps are
  {
		$ar[] = $stamp;
		if (!empty($stamp->startdate))
			$firstdate = min($stamp->startdate, $firstdate);           
		$firstdate = min($stamp->enddate, $firstdate); 
		$lastdate = max($stamp->enddate, $lastdate); 
  }

	if (!empty($node->firstdate))           // if the user has already decided on the
		$firstdate = $node->firstdate ;       // override our discovered

  $y = format_date($firstdate ,  'custom', 'Y');
//  drupal_set_message('first date '. $y, 'info');
         
  foreach ($ar as $stamp) 
  {
  	
		$total = 0 ; 
//		$amount = _evaluate($node->reading);
		
		$rows[] = array(
			format_date($stamp->enddate, "small"),
			carbon_link_from_title($stamp),
      $stamp->sourcemodel,
			l($stamp->sourcetitle, 'node/'.$stamp->sourceid),					
      $stamp->reading.' '._get_scope_key($stamp->scope),            
			$stamp->units,
			$stamp->adjustment,
//			sprintf("%.0f",$stamp->$total)
			);
  }  
  
  $output = "<h1 class='title'>All Carbon Stamps</h2>" ;
  $output .= theme('table', $header, $rows); 

	
	for ($i = 0 ; $i < 20 ; $i++)
	{	
		$y = format_date($firstdate ,  'custom', 'Y');
    $m = format_date($firstdate ,  'custom', 'm');
    $d = format_date($firstdate ,  'custom', 'd');
		$firstdayinyear = mktime(0, 0, 0, $m, $d, $y+$i); 
		$firstdayinnextyear = mktime(0, 0, 0, $m, $d, $y+$i+1);
		$output .= carbon_calculate($ar, $firstdayinyear, $firstdayinnextyear);
		if ($firstdayinnextyear> $lastdate) 
			break ; 
	}  	
 
	
//  $output .= theme('pager', NULL, 50, 0);

  $output .= '<p></p>';
//  $output .= '<h2 class="title">Carbon stamps included in this footprint</h2>';
//  $output .= carbon_stamp_view_linked($node->nid); 
  
  return $output;
}



function theme_carbon_footprint_page($form) {

    return "abcdef";
}


/**
 * Implemenation of hook_load 
 * @param node object to load additional information for
 * @return object with carbon fields 
 */
function carbon_footprint_load($node)  {
  global $user ; 
  $query = db_query("SELECT f.*, s.uid as friends_uid " .
    "FROM {carbon_footprint} f " .
    "LEFT  JOIN {carbon_footprint_share} s ON  f.nid = s.nid " . 
    "WHERE f.nid = %d ",
    $node->nid);
    
  // most user will not be sharing records. However if they are 
  // we pick up their friends uid's while loading the footprint 
  $first_record = null ; 
  while ($record = db_fetch_object($query))
  {
    if (!isset($first_record))
      $first_record = $record ;   
      $first_record->friends = array(); 
      
    if (isset($friends_uid))
      $first_record->friends[] = $friends_uid ;     	
  }
	$result = carbon_stamp_array_load($node->nid, 0, '');
	$first_record->stamp = $result['stamp'];
	$first_record->source = $result['source'];

//	drupal_set_message('loaded '.count($result['stamp']). ' stamps for '.$node->nid, 'info'); 

	return $first_record;
}



function carbon_footprint_submit(&$form)
{
	$sources = carbon_source_fetch_array() ;
	$source_map = _get_source_map($sources);
	$model_map = _get_model_map($sources);
	$model_keys = array_keys($model_map);

	if (!empty($form->firstdate_s))
	{
		$form->firstdate = strtotime($form->firstdate_s) ;
	}
	
	if (isset($form->model_index))
	{
		$form->model = $model_keys[$form->model_index] ;
	}
}



 
function carbon_footprint_handle_updates(&$node, $sources) 
{		
	global $user ; 
	$prefix = 'form.' ;	
	$source_keys = array_keys(_get_source_map($sources));

//	drupal_set_message("number of sources is " .sizeof($source_keys) , 'info');

	// it's bad that we we have load these stamps again
	if (!isset($node->stamp))
	{
		$result = carbon_stamp_array_load($node->nid, 0, '');
		$node->stamp = $result["stamp"];
//		$node->source = $result['source'];		
	}
//	drupal_set_message('loaded '. count($source_keys). ' sources '.
//								count($node->stamp) . ' stamps ', 'info');

	for ($i=0 ; ; $i++)
	{
		$form = new Node ;
		_map_stamp_fields($form, $node, $i, $source_keys);

		// test for a combination of any fields that must 
		// be required to do an update
				
		if (empty($form->cloneid) && !isset($form->code_index))
			break ; 	// probably no other fields to update. 

		if (empty($form->nid))
		{		
			if (!empty($form->reading))
			{
				// stamp inserts 	
				$stamp = new Node;	
				if (isset($form->cloneid))
				{
					if (!isset($node->stamp[$form->cloneid]))
					{
						drupal_set_message("cannot clone, copy of existing stamp not available","info");
						continue ;
					}
					$template = $node->stamp[$form->cloneid] ; 
					$stamp->title = $template->title ; 
					$stamp->code = $template->code;	
					$stamp->scope = $template->scope;	
					$stamp->startdate = $template->startdate;						
					$stamp->adjustment = $template->adjustment;	
					$stamp->protection = $template->protection; 
					$stamp->exclude = $template->exclude;					
					$stamp->status = 1; 
				}
				else
				{
					$stamp->protection = 0 ; 
					$stamp->exclude = false ;		
          $stamp->adjustment =  1 ; 			
				}
        // ask carbon_stamp_insert() (invoked by node_save) to attach to this footprint
				$stamp->fid_delete = array();
        $stamp->fid_insert = array($node->nid) ; 
				$stamp->uid = $node->uid ;      // inherit from footprint 	
				$stamp->type = "carbon_stamp" ;				
				$stamp->status = 1 ;
				
				// now override all the values provided in the form
				
				_map_stamp_fields($stamp, $node, $i, $source_keys);
				node_save($stamp);
      	drupal_set_message("Added ".$stamp->title.', '.$stamp->code.'='.$stamp->reading, "info");	
			}
		}	
		else
		{
			// this is an existing stamp, we are just tweaking a few fields 
			// get the original node from the footprint.
			

			if (empty($node->stamp[$form->cloneid]))
			{
				drupal_set_message("cannot update, copy of existing stamp not available to compare","info");
				continue ;
			}

			$stamp = $node->stamp[$form->nid] ; 
			if ( $stamp->reading != $form->reading ||   
				   $stamp->enddate != $form->enddate
				 )
			{
				_map_stamp_fields($stamp, $node, $i, $source_keys);
				node_save($stamp);
	      drupal_set_message("Updated ".$stamp->title.', '.$stamp->code.'='.$stamp->reading, "info");
			}		
		}			
	}
}


/**
 * Implementation of hook_insert, which saves carbon-specific information
 * into the carbon table
 * @param node object
 */

function carbon_footprint_insert(&$node) 
{		
  db_query("INSERT INTO {carbon_footprint} (postcode, organization, model, firstdate, " .
    		"filing_status, protection, nid) " .
    		"VALUES ('%s', '%s', '%s', %d, %d, %d, %d)", 
	$node->postcode, $node->organization, $node->model, $node->firstdate,
	$node->filing_status, $node->protection,$node->nid);
  drupal_set_message(t("To update your footprint now click on " .
        "the <em>edit</em> tab. To retrieve your footprint on a later " .
        "date click on <em>my account</em>."));
        
  $sources = carbon_source_fetch_array() ;
  carbon_footprint_handle_updates($node, $sources); 
}

/**
 * Implementation of hook_update, which saves updated todo-specific 
 * information into the todo table
 * 
 * We have to carefully step through the submitted form and compare
 * the values in the form with the values in the array 'ar''
 * @param node object
 */
function carbon_footprint_update(&$node) 
{			
  db_query("UPDATE {carbon_footprint} " .
    		"SET postcode='%s', organization='%s', model='%s', firstdate=%d, " .
    		"filing_status=%d, protection=%d WHERE nid=%d", 
	$node->postcode, $node->organization, $node->model, $node->firstdate,
	$node->filing_status, $node->protection,$node->nid);

  $sources = carbon_source_fetch_array() ;
  carbon_footprint_handle_updates($node, $sources);  
}


/**
 * Implementation of hook_delete().
 */
function carbon_footprint_delete($node) {
  db_query("DELETE FROM {carbon_stamp_footprint} WHERE fid = %d", $node->nid); 
  db_query("DELETE FROM {carbon_footprint_share} WHERE nid = %d", $node->nid); 
	db_query("DELETE FROM {carbon_footprint}       WHERE nid = %d", $node->nid);
}


/**
 * compress all the emission properties into a string as 
 * key value pairs and store in a single field 
 */ 

function _implode_data(&$node)
{
	$data = "" ; 
	$key = _data_type();
	
	foreach ($key as $entry) 
	{
		if (strlen($node->$entry) > 0)
			$data .= $entry.":".$node->$entry.";"; 
	}
	return $data ; 
}


?>