<?php
/** 
 * $Id: carbon_source.inc,v 1.5 2006/12/20 13:37:49 johnackers Exp $
 *
 * created on 11-Nov-2006
 *
 * @file 
 * definition of carbon_source node. Each carbon_source 
 * node contains parameters that allow the carbon dioxide 
 * emissions to be calulated from a particular soutrce of 
 * emissions e.g. domestic gas supply, car miles etc. 
 * 
 * This is a very straightforward standard drupal node 
 * implementation. 
 *  
 */


/**
 * Implementation of hook_access().
 * 
 * Refer to carbon_access because it is easier to check 
 * access for all carbon types in one place.  
 */


function carbon_source_access($op, $node)
{
	return carbon_access($op, $node);
}




/**
 * Render a page listing a one line summary of all carbon sources 
 * 
 * TODO add edit operations
 */

function carbon_source_page() {
  	global $user;

  $header = array(
//    array("data" => t("created")),
    array("data" => t("model"), "field" => "model"),
    array("data" => t("code"), "field" => "code"),
    array("data" => t("title"), "field" => "title"),
    array("data" => t("units"), "field" => "units"),
    array("data" => t("class"), "field" => "classname"),
    array("data" => t("multiplier"), "field" => "multiplier"),
    array("data" => t("params"))
  );
  
  $sql = _get_carbon_source_sql();

  $sql .= tablesort_sql($header); 
  
  // get the links (no range limit here)
  $queryResult = db_query($sql);
  $rows[] = array();
  $protection = _get_protection_options();

  while ($node = db_fetch_object($queryResult)) {
 
//	 $access_ro = carbon_access('view', $node);
    $access_rw = carbon_access('update', $node, $node->uid);
	  $link = $access_rw ? l($node->code,'node/'.$node->nid.'/edit') : $node->code; 


	  $rows[] = array(
//					format_date($node->created,'small'),
					$node->model,
					$link,
					$node->title, 
					$node->units,
					$node->classname,
					$node->multiplier,
					$node->params										
					);
 	}

  $output = theme('table', $header, $rows);
  $output .= theme('pager', NULL, 10, 0);

  if (carbon_access('create', 'carbon_source'))
    $output .= l("create new source","node/add/carbon_source");
  return $output;
}

/**
 * hold the list of available sources globally because
 * different functions use this array during the same 
 * transaction. 
 */


function carbon_source_fetch_array()
{ 
  global $source ;
  if (empty($source))
  {
    $queryResult = db_query(_get_carbon_source_sql());
    $source = array(); 
    
    while ($node = db_fetch_object($queryResult)) 
    {
     $source[] = $node ; 
    }
  }
  return ($source); 
}

/**
 * The administrator can select which carbon models to 
 * make available. Every select for a carbon source 
 * is appended with the system wide list of approved 
 * models. 
 */

function _get_carbon_source_sql()
{
  $sql = "SELECT r.*, n.* FROM {carbon_source} r " .
  "LEFT JOIN {node} n ON r.nid = n.nid " .  
  "INNER JOIN {users} u on n.uid = u.uid " ;
  
  $model = variable_get("carbon_models", "");
  if (!(empty($model)))
  {
    $models = explode(",", $model);

    // wrap each element with " "
    for ($i = 0 ; $i < count($models) ; $i++) $models[$i] = "'".trim($models[$i])."'";
        $model = implode(",", $models); 

    $sql .= " WHERE model in (".$model.") " ; 
  }    
  return $sql ;     
}



/**
 * returns a map of key=> value pairs where the 
 * key is the code in the database and 
 * and the value is text presented to the user
 * 
 * note that adding sources might while a user 
 * is selecting might result in the wrong option. 
 */ 

function _get_source_map($source)
{
	$map = array(); 
	foreach($source as $s)
	{
			$map[$s->code] = $s->title. ', ' . $s->units ;
	}
	ksort($map); 
	return $map ; 
}


/**
 * @return a map of key=> value pairs where the 
 * key is the code in the database and 
 * and the value is text presented to the user
 * 
 * note that adding sources might while a user 
 * is selecting might result in the wrong option. 
 */ 

function _get_model_map($source)
{
	$map = array(); 
	$map[''] = '-no preference-' ;
	foreach($source as $s)
	{
			$map[$s->model] = $s->model ;
	}
	ksort($map); 
	return $map ; 
}




/**
 * Implementation of hook_form().
 */



function carbon_source_form(&$node, &$param) {
  global $user;

  $form["title"] = array(
	"#type" => "textfield", 
    "#title" => "title", 
    "#required" => TRUE, 
    "#size" => 20,
    "#maxlength" => 80,       
    "#weight" => -2,
    "#default_value" => $node->title, 	
    "#description" => "Full descriptive name of this source");

  $form["units"] = array(
	"#type" => "textfield", 
    "#title" => "units", 
    "#required" => TRUE, 
    "#weight" => -1,
    "#default_value" => $node->units, 
    "#size" => 20,
    "#maxlength" => 20,     	
    "#description" => "Should be fairly short, more details can be provided in notes");


  $form["code"] = array(
	"#type" => "textfield", 
    "#title" => "code", 
    "#required" => TRUE, 
    "#weight" => -1,
    "#size" => 20,
    "#maxlength" => 20,   
    "#default_value" => $node->code, 	
    "#description" => "Short unique code used to define this source. " .
    		"This code must match the same code in other models if you want " .
    		"to compare the carbon footprint calculated by two models from " .
    		"the same inputtted values");
    
  $form["model"] = array(
	"#type" => "textfield", 
    "#title" => "model", 
    "#required" => TRUE, 
    "#size" => 20,
    "#maxlength" => 20,   
    "#weight" => 0,
    "#default_value" => $node->model, 	
    "#description" => "The overall model e.g. crag, carbon coach");

  $form["multiplier"] = array(
	"#type" => "textfield", 
    "#title" => "conversion factor", 
    "#required" => TRUE, 
    "#size" => 20,
    "#maxlength" => 20,       
    "#weight" => 0,
    "#default_value" => $node->multiplier, 	
    "#description" => "Conversion factor");

  $form["classname_index"] = array(
	"#type" => "select", 
    "#title" => "classname", 
    "#required" => TRUE, 
    "#size" => 3,
    "#maxlength" => 20,       
    "#weight" => 0,
    "#options" => _get_source_class_key(null),
    "#default_value" => array_search($node->classname, _get_source_class_key(null)),
    "#description" => "PHP class that calculates the emissions");


  $form["params"] = array(
	"#type" => "textarea", 
    "#title" => "params", 
    "#required" => false,
    "#size" => 20,
    "#maxlength" => 80,        
    "#weight" => 1,
    "#height" => 2,
    "#default_value" => $node->params, 	
    "#description" => "Additional parameters used by the class");


  $form["body"] = array(
	"#type" => "textarea", 
    "#title" => "notes", 
    "#required" => false, 
    "#size" => 4,
    "#maxlength" => 20,       
    "#weight" => 1,
    "#default_value" => $node->body, 	
    "#description" => "Notes about this carbon source and how the conversion " .
    		"factors have been calculated");

  return $form;
}


/**
 * Validate our forms
 * @param string form id
 * @param array form values
 */
function carbon_source_validate($form_id, $form_values) 
{

}


/**
 * Implemenation of hook_load 
 * @param node object to load additional information for
 * @return object with carbon fields 
 */
function carbon_source_load($node)  {
		
	$record = db_fetch_object(db_query("SELECT * " .
			"FROM {carbon_source} WHERE nid = %d ", $node->nid));
	return $record;
}


function carbon_source_submit(&$node)  {
		
	$node->classname = _get_source_class_key($node->classname_index);
}


/**
 * Implementation of hook_insert, which saves carbon-specific information
 * into the carbon table
 * @param node object
 */
function carbon_source_insert(&$node) {

    db_query("INSERT INTO {carbon_source} " .
    		"(nid, model, code, units, classname, multiplier, params)" .
    		"VALUES (%d, '%s', '%s', '%s', '%s', %f, '%s' )", 
	$node->nid, $node->model, $node->code, $node->units, 
	$node->classname, $node->multiplier, $node->params);
}


/**
 * Implementation of hook_update, which saves updated todo-specific 
 * information into the todo table
 * @param node object
 */
function carbon_source_update(&$node) {
	
  db_query("UPDATE {carbon_source} set model='%s',code='%s',units='%s'," .
    		"classname='%s', multiplier=%f, params='%s' WHERE nid=%d", 
	$node->model, $node->code, $node->units, $node->classname, 
	$node->multiplier, $node->params, $node->nid);
}


/**
 * Implementation of hook_delete().
 * (not certain that this is necessary)
 */
function carbon_source_delete($node) {
  db_query("DELETE FROM {carbon_source} WHERE nid = %d", $node->nid);
}



/**
 * Implementation of hook_view, add our node specific information
 * @param node object to display
 * @param boolean is this a teaser or full node?
 * @param boolean is this displaying on its own page
 */
function carbon_source_view(&$node, $teaser = FALSE, $page = FALSE) 
{
//	if (!user_access('view carbon sources')) {
//		return true;
//	}

    $node = node_prepare($node, $teaser);
    $carbon_source_info = theme('carbon_source_table_view', $node);  // try item_list
    $node->body .= $carbon_source_info;
    $node->teaser .= $carbon_source_info;
}

/**
 * Theme function to display additional node data
 * @param node to display
 * @return HTML string with additional node information
 */
 

// then create an array of column names

function theme_carbon_source_table_view($node)
{
  $header = array(
    array("data" => t("created")),
    array("data" => t("model")),
    array("data" => t("code")),
    array("data" => t("units")),
    array("data" => t("classname")),
    array("data" => t("multiplier")),
    array("data" => t("params"))
  );
	

  $rows[] = array(
					format_date($node->created, 'small'),
					$node->model,
					$node->code,
					$node->units,
					$node->classname,
					$node->multiplier,
					$node->params										
					);	

  $output = theme('table', $header, $rows);
//  $output .= theme('pager', NULL, 50, 0);

  return $output;
}


/**
 * @return the class for a given index
 */

function _get_source_class_key($i)
{
  global $source_class_keys ; 
  if (empty($source_class_keys))
    $source_class_keys = array_keys(carbon_calculate_class_map()); 
  if (!isset($i)) return $source_class_keys ; 
  return $source_class_keys[$i] ;
}

?>
