<?php
/** 
 * $Id: carbon_stamp.inc,v 1.8 2006/12/20 13:30:13 johnackers Exp $ 
 * 
 * Drupal carbon stamp module 
 *
 * Created on 18-Oct-2006
 *
 */


/**
 * Render a page listing a one line summary of all carbon footprints 
 * TODO add edit operations
 */


function carbon_stamp_page($uid) {
  global $user;

  $header = array(
    array("data" => t("title"),   "field" => "title"),
    array("data" => t("start"),   "field" => "startdate"),
    array("data" => t("end"),     "field" => "enddate"),
    array("data" => t("code"),    "field" => "code"),
    array("data" => t("reading"), "field" => "reading"),
    array("data" => t("fp's"),    "field" => "numfoot"),
    array("data" => t("status"),  "field" => "status"),
    array("data" => t("filed by"),"field" => "name",)
//	,t('Operations')
  );

  $sql = "SELECT  t.*, n.title, n.nid, n.status, n.uid, n.type, name, count(*) as numfoot, j.sid " .
  		   "FROM {carbon_stamp} t " .
         "INNER JOIN {node} n                   ON t.nid = n.nid " .
         "INNER JOIN {users} u                  ON n.uid = u.uid " .
         "LEFT  JOIN {carbon_stamp_footprint} j ON t.nid = j.sid " .         
         ((empty($uid) ? '' : " where n.uid = ".$uid. " ") . 
         "GROUP BY t.nid "); 
  
  $sql .= tablesort_sql($header); 
  
  // get the links (no range limit here)
  $queryResult = db_query($sql);
	
  while ($node = db_fetch_object($queryResult)) {
    
    $rows[] =  array(carbon_link_from_title($node),
				$node->startdate == 0 ? '' : format_date($node->startdate, "small"),
				format_date($node->enddate,"small"),
				$node->code, 
				$node->reading.' '._get_scope_key($node->scope),
				$node->numfoot <= 1 ? (isset($node->sid) ? 1 : 0) : $node->numfoot, 		
				$node->status >= 1 ? 'pub' : '',
				$node->name);
  }

  $output = theme('table', $header, $rows);
  $output .= theme('pager', NULL, 10, 0);

  return $output;
}

/**
 * Implementation of hook_access().
 * 
 * Delegate carbon_access to the carbon module because it 
 * is easier to check access for all carbon types in one place.  
 */

function carbon_stamp_access($op, $node)
{
	return carbon_access($op, $node);
}

/**
 * 
 * This is the single most important function as the select statement identifies 
 * joins the recorded stamps with the available carbon sources. 
 * 
 * Note that some source fields have been renamed e.g. sourceid, sourceclass, sourcetitle 
 *
 *@param $nid  select stamps with this id, can be null
 *@param $uid  select stamps with this uid, can be null  
 *@return an array of carbon stamps (and an array of carbon sources) 
 */


function carbon_stamp_array_load($nid, $uid, $org)
{
	$query ="SELECT  nstamp.title, nstamp.status, nstamp.uid, nstamp.type, stamp.*, s.uid AS shared_uid, " .
		"nsource.title as sourcetitle, source.nid as sourceid, source.classname as sourceclass, " .
		"source.code, source.model as sourcemodel, units, multiplier, params, " .
    "f.model as preferred_model, f.organization " .
		"FROM {carbon_stamp} stamp " .
    "LEFT JOIN {carbon_source} source ON stamp.code=source.code " .
		"LEFT JOIN {carbon_stamp_footprint} j ON stamp.nid = j.sid " .	
    "INNER JOIN {carbon_footprint} f      ON     f.nid = j.fid " .
    "LEFT JOIN {carbon_footprint_share} s ON     f.nid = s.nid " . 
    "INNER JOIN {node} nstamp ON  stamp.nid = nstamp.nid ". 
    "INNER JOIN {node} nsource ON  source.nid = nsource.nid ".            		
		"WHERE nsource.status ".     // source must be published   
    "AND   nstamp.status ".      // stamp must be published  
    "AND (j.fid = %d  " .           (empty($nid) ?  " OR 1) " : ") ").
    "AND (nstamp.uid = %d " .       (empty($uid) ?  " OR 1) " : ") ").           
    "AND (f.organization = '%s' " . (empty($org) ?  " OR 1) " : ") ").  
    "" ;   

//	$query = db_rewrite_sql($query); 
//  drupal_set_message(" my sql ". $query, "info"); 

	$queryResult =  db_query($query, $nid, $uid, $org);

	$stamps = array(); 
	
	$sources = array();
  $num = 0 ; 
	while ($record = db_fetch_object($queryResult))
	{
    $num++ ; 
		if ($record->nid != null)
    {
//      drupal_set_message("pref model ". $record->preferred_model. "  src model ". $record->sourcemodel, "info");   
      // we don't join the footprint preferred model with the source model because the
      // source may not be available for the selected model. Instead walk through the rows 
      // and pick the first row that matches by code only and replace that with a model 
      // match if it happens to occur.   
      if ($record->sourcemodel == $record->preferred_model || !isset($stamps[$record->nid]))
		    $stamps[$record->nid] = $record ;            
    }		
		$sources[$record->nid] = $record ;
	}   
	return array("source" => $sources, "stamp" => $stamps);
}


function _stamp_form(&$form, $source_keys, $source_descriptions, $stamp, $i, $group, $group_title)
{
	  $stamp_form = "form.".$i ; 
	  $stamp_group_form = "form.".$group ; 

	  if ($form[$stamp_group_form] == null)
	  {
		  $form[$stamp_group_form] = array(
				"#type" => "fieldset", 
				"#title" => $group_title, 
	    		"#weight" => 1,				
				"#collapsible" => TRUE,
				"#collapsed" => false);
	  }

	  $form[$stamp_group_form][$stamp_form] = array(
			"#type" => "fieldset", 
			"#collapsible" => TRUE,
			"#collapsed" => false);

	  $form[$stamp_group_form][$stamp_form]["form.".$i.".title"] = array(
		  "#type" => "textfield", 
    	"#title" => t("title"),  
	    "#required" => false, 
 	    "#default_value" => empty($stamp->title) ? "My carbon emissions" : $stamp->title ,
      "#size" => 30,
      "#maxlength" => 40, 
	    "#weight" => -2,
	    "#description" => "Name of house or car or trip destination");


	  $form[$stamp_group_form][$stamp_form]["form.".$i.".scope_index"] = array(	    
		  "#type" => "radios", 
	    "#title" => t("this carbon debit"), 
	    "#required" => TRUE, 
		  "#options" => _get_scope_description(),
      "#size" => 14,
      "#maxlength" => 20, 
	    "#weight" => -1,
	    "#default_value" => empty($stamp->scope) ? 0 : $stamp->scope, 	
	    "#description" => "Note that meter readings are subtracted from one another to calculate " .
	    		"consumption. You need to have at least two meter readings before this source of " .
	    		"emissions will be included in your carbon footprint. ");	    
	    
	  $form[$stamp_group_form][$stamp_form]["form.".$i.".code_index"] = array(
		  "#type" => "select", 
    	"#title" => t("select type of carbon debit"),  
	    "#required" => TRUE, 
		  "#options" => $source_descriptions,   
	    "#default_value" => empty($stamp->code) ? 0 : array_search($stamp->code,$source_keys),
      "#size" => 1,
      "#maxlength" => 20, 
	    "#weight" => 0,
	    "#description" => "Select the carbon source with the units that match");
  	  
	  $form[$stamp_group_form][$stamp_form]["form.".$i.".reading"] = array(
		  "#type" => "textfield", 
    	"#title" => t("amount"),  
	    "#required" => FALSE, 
	    "#default_value" => empty($stamp->reading) ? "" : $stamp->reading, 
      "#size" => 14,
      "#maxlength" => 20, 
	    "#weight" => 0,
	    "#description" => "Number of units, miles, gallons, litres, KwH etc.") ;

	  $form[$stamp_group_form][$stamp_form]["form.".$i.".startdate_s"] = array(
		  "#type" => "textfield", 
    	"#title" => t("starting date of emissions period"),  
	    "#required" => FALSE, 
	    "#default_value" => empty($stamp->startdate)  ? "" : format_date($stamp->startdate, "custom", "Y-m-d H:i:s O"),
      "#size" => 14,
      "#maxlength" => 20, 
	    "#weight" => 1,
	    "#description" => 
	    		t("Format: %time. ", 
	    		array("%time" => format_date(time(), "custom", "Y-m-d H:i:s O"))).
	    		t("Normally leave blank. However If these emissions " .
	    		"have occurred over a period of time enter the starting date. " .
	    		"This might be the first date of a daily or monthly trip."));



	  $form[$stamp_group_form][$stamp_form]["form.".$i.".enddate_s"] = array(
		  "#type" => "textfield", 
    	"#title" => t("date (or end date) of emission"),  
	    "#required" => FALSE, 
	    "#default_value" => $stamp->enddate == 0 ? "" : format_date($stamp->enddate, "custom", "Y-m-d H:i:s O"),
      "#size" => 14,
      "#maxlength" => 20, 
	    "#weight" => 2,
	    "#description" => 
	    		t("Format: %time. Leave blank to use today's date. ", 
	    		array("%time" => format_date(time(), "custom", "Y-m-d H:i:s O"))).
	    		t("The date of the emissions (or if you specified a start date " .
	    		"above, the date on which the emissions ceased)"));

	     
	  $form[$stamp_group_form][$stamp_form]["form.".$i.".adjustment"] = array(
		  "#type" => "textfield", 
    	"#title" => t("adjustment"),  
	    "#required" => FALSE, 
	    "#default_value" =>  $stamp != null && $stamp->adjustment != 0 ? $stamp->adjustment : 1, 
	    "#weight" => 3,
      "#size" => 10,
      "#maxlength" => 20, 
	    "#description" => t("This is the complicated bit. Normally 1 except when " .
	    		"<h4>Two way trip</h4>Enter <b>2</b> if this is a two way trip. " .
	    		"<h4>Sharing heating, electricity or a car trip</h4>" .
	    		"For example if you are sharing with 4 people you might enter <b>1/4</b>. " .
	    		"<h4>Frequent Trip</h4>If you make this trip 1 day a month, you might " .
	    		"enter <b>1/30</b>, or if its once a year you might enter <b>1/365</b>. " .
	    		"If this is a daily commute to work and you take 5 weeks holiday, you " .
	    		"could be very precise and enter <b>(5/7)*(47/52)</b>. " .
	    		"Don\"t forget to enter the starting date of these trips above")); 
		     
	    
	  $form[$stamp_group_form][$stamp_form]["form.".$i.".nid"] = array(
		  "#type" => "hidden", 
	    "#value" => $stamp->nid);	

	  $form[$stamp_group_form][$stamp_form]["form.".$i.".nid"] = array(
		  "#type" => "hidden", 
	    "#value" => $stamp->nid);	
	    
	  $form[$stamp_form]["form.".$i.".code"] = array(
		  "#type" => "hidden",
	    "#value" => null);
	
	return $form ; 
}

/**
 * $stamp is the actual stamp that we are trying to edit (null if creating)
 * $clone is the stamp that we want to clone and add as a new entry 
 * but with, say, a different meter reading 
 */


function _stamp_repeating_form(&$form, $stamp, $clone, $i, $group)
{
	  $group_title = $stamp->title ; // . " | " . $clone->code ;	

	  $stamp_group_form = "form.".$group ; 
    $stamp_form = "form.".$i ; 
	  if ($form[$stamp_group_form] == null)
	  {
		  $form[$stamp_group_form] = array(
				"#type" => "fieldset", 
				"#title" => $group_title,
        "#weight" => 0,	    					 
				"#collapsible" => TRUE,
				"#collapsed" => true);
	  }

	  if ($form[$stamp_group_form][$stamp_form] == null)
	  {
		  $form[$stamp_group_form][$stamp_form] = array(
				"#type" => "fieldset", 
//          "#title" => $title, 
//          '#prefix' => '<div class="container-inline">',
//          '#suffix' => '</div>',	
				"#collapsible" => false,
				"#collapsed" => false);
	  }

	  $form[$stamp_group_form][$stamp_form]["form.".$i.".reading"] = array(
		  "#type" => "textfield",  
      "#title" => (isset($clone->scope) && $clone->scope == 1) ? "meter reading" : "amount",
	    "#required" => false, 
	    "#default_value" =>  empty($stamp) ?  "" : $stamp->reading, 
      "#size" => 20,
      "#maxlength" => 20,    
	    "#weight" => -1,
      "#description" => empty($clone) ? "amount" : $clone->sourcetitle. ", ".$clone->units,  );
	    

	  $form[$stamp_group_form][$stamp_form]["form.".$i.".enddate_s"] = array(
		  "#type" => "textfield", 
    	"#title" => t("date (or end date) of emission"),  
    	"#length" => 10,      	
	    "#required" => false, 
	    "#default_value" => empty($stamp->enddate) ? "" : format_date($stamp->enddate, "custom", "Y-m-d H:i:s O"),
	    "#weight" => 0,
      "#size" => 10,
      "#maxlength" => 20,   
	    "#description" => $stamp != null ? "" : t("Format: %time. Leave blank to use today's date.", array("%time" => format_date(time(), "custom", "Y-m-d H:i:s O"))));

	// all the other fields will be copied from the stamp with nid = "cloneid"		
		
	  $form[$stamp_group_form][$stamp_form]["form.".$i.".cloneid"] = array(
		  "#type" => "hidden", 
	    "#value" => $clone->nid);

	  $form[$stamp_group_form][$stamp_form]["form.".$i.".nid"] = array(
		  "#type" => "hidden", 
	    "#value" => $stamp->nid);
	    	
	   return $form ; 
}

/**
 * when a user creates or edits a carbon_stamp, s/he may want to 
 * add or remove it from a particular carbon footprint. This 
 * form is only attached when inidvidual carbon stamps are 
 * edited.
 */

function _add_footprint_select(&$form, $node, $i, $group)
{
  global $user ; 
  $stamp_group_form = "form.".$group ; 
  $stamp_form = "form.".$i.".footprint" ; 

  // find all the carbon footprints owned by the same user as this stamp
  // or , if this is a new stamp, the logged in user. This is really to
  // make testing easier.

  $uid = isset($node->uid) ? $node->uid : $user->uid ;  
  $queryResult = db_query("SELECT n.nid,n.title from {node} n ".
        " INNER JOIN {carbon_footprint} f ON n.nid=f.nid ".
        " WHERE type='carbon_footprint' and uid = ".$uid);  
    
  $options = array();  

  if (isset($node->fid_map)) 
    $ticked_boxes = array_keys($node->fid_map);
    
  $nid = array();   
  while ($record  = db_fetch_object($queryResult)) {
    
    $nid[] = $record->nid ; 
    $options[$record->nid] = $record->title ;
  }
  // this clumsy code sets the only check box if this is a new 
  // carbon stamp and there is only one footprint to attach to
    
  if (!isset($ticked_boxes) && count($nid) == 1)
  {
    $ticked_boxes = $nid ;
  } 
    
  $form[$stamp_group_form][$stamp_form]["fid_updated"] = array(
  "#type" => "checkboxes",
  "#title" => t("select footprint(s)"),
  "#default_value" => $ticked_boxes,
  "#options" => $options, 
  "#description" => t("Click on the footprints to which you would like to attach this carbon stamp.")
  );    
}



function carbon_stamp_form(&$node, &$param) {
	global $user;

	$source = carbon_source_fetch_array();
	$map = _get_source_map($source);
	$source_keys = array_keys($map); 
	$source_descriptions = array_values($map); 
  $form = array(); 
  $form = _stamp_form($form, $source_keys, $source_descriptions, $node, 0, 0, $node->title);
  _add_footprint_select($form , $node, 0 , 0 ); 
  return $form ;   
}

/**
 * scope is held as a integer in the database which 
 * indexes this map directly, so take care about changing the 
 * order of the entries in the table. 
 */

function _scope_map()
{
  return array(
    "" =>    t("is a one off"),
    "m" =>   t("is a meter or mileometer reading"),
    "r" =>   t("repeats daily")
    );
}

/**
 * @return a short code/description of this numeric scope value
 */

function _get_scope_key($i)
{
  global $scope_keys ; 
  if (empty($scope_keys))
    $scope_keys = array_keys(_scope_map()); 
  return $scope_keys[$i] ;
}

/**
 * @return a long description of this numeric scope value
 */

function _get_scope_description($i = null)
{
  global $scope_values ; 
  if (empty($scope_values))
    $scope_values = array_values(_scope_map());   
  return $i == null ?   $scope_values : $scope_values[$i] ;
}

/**
 * @return a short description of this numeric scope value
 */


function _get_scope_short_description($i = null)
{
  if ($i == 0) return null ; 
  return _get_scope_description($i); 
}


/**
 * Validate our forms
 * @param string form id
 * @param array form values
 */
function carbon_stamp_validate($form_id, $form_values) 
{
	for ($i = 0 ; $i < 500 ; $i++)
  {
	  _validate_adjustment($form_id, "form.". $i .".adjustment");
    _validate_date($form_id, "form.". $i .".startdate_s"); 
    _validate_date($form_id, "form.". $i .".enddate_s"); 
//    _validate_date($form_id, "form.". $i .".firstdate"); 
  }
}


/**
 * the field is evaluated PHP so it is important not to 
 * allow any code to be maliciously inserted.
 */

function _validate_adjustment($form_id, $field_id)
{
  $matches = array();
  if (!empty($form_id->$field_id))  
    if (!preg_match('/^[0-9\.\*\/\+\-\(\)]*$/', $form_id->$field_id, $matches))
      form_set_error($field_id, t("Only . 0-9 ( ) * / + or - accepted in 'adjustment' field"));  	
}


function _validate_date($form_id, $field_id)
{
  $matches = array();
  if (!empty($form_id->$field_id))  
    if (!preg_match("/^\d\d\d\d\-\d\d\-\d\d/", $form_id->$field_id, $matches))
      form_set_error($field_id, t("Please check that date is in YYYY-mm-dd format.")); 
}



/**
 * This is a hook used by node modules. It is called after validation has 
 * succeeded and before insert/update. It is used to for actions which must 
 * happen only if the node is to be saved.
 */
function carbon_stamp_submit(&$node)
{			
	$source_keys = array_keys(_get_source_map(carbon_source_fetch_array()));
	_map_stamp_fields($node, $node, 0, $source_keys);
//  drupal_set_message()

  global $fid_map ; 
  if (!isset($fid_map))
    $fid_map = array();
     
  // after the editing, on checked boxed the values in 
  // fid_updated are set to the keys, on unchecked boxes
  // the values are set to zero.  

  $fid_updated = array_flip($node->fid_updated);
  if (isset($fid_updated[0])) 
    unset($fid_updated[0]);   // loose the 0 key created by any unchecked boxes
  $node->fid_insert = array_diff(array_keys($fid_updated), array_keys($fid_map));
  $node->fid_delete = array_diff(array_keys($fid_map), array_keys($fid_updated));  
}

/**
 * This function called locally and from carbon_footprint 
 * after footprint editing. 
 */

function _map_stamp_fields(&$dest, $src, $i, $source_keys)
{
	$fields = array("nid","cloneid","code_index","scope_index",
	"title","reading","adjustment","startdate_s","enddate_s","exclude");		
	
	// the source fields are mapped in a wack way so 
	// them onto the the destination object

	foreach ($fields as $f)
	{
		_map_form_field($dest, $src, $i, $f);
	}	

	$upd = 0 ;
	if (isset($dest->code_index))
	{
		$dest->code = $source_keys[$dest->code_index];  $upd += 1;
	}
	if (isset($dest->scope_index))
	{
		$dest->scope = $dest->scope_index;  $upd += 2;
	}
	if (!empty($dest->startdate_s))
	{
		$dest->startdate = strtotime($dest->startdate_s);  $upd += 4;
	}
	if (!empty($dest->enddate_s))
	{
		$dest->enddate = strtotime($dest->enddate_s);	$upd += 8;
	}



	if (empty($dest->enddate))
	{
		$dest->enddate = time();   $upd += 16;
	}
//	drupal_set_message('(debug) updated fields map :'.$upd, 'info'); 
}


function _map_form_field(&$dest, $src, $i, $field)
{
	$prefix = 'form.' ;	
	$fullname = $prefix.$i.'.'.$field ;

	_set_field($dest, $field, $src, $fullname); 

}


function _set_field(&$dest, $name, $src, $fullname)
{
	if (isset($src->$fullname))
		$dest->$name = $src->$fullname ; 			
	else 
	// and if the field is not present, set it (to null) but 
	// if the field has a non null value don't overide it with a null
		
		if (!isset($dest->$name))
			$dest->$name = $src->$fullname ; 					
}

/**
 * Implementation of hook_load.
 * Slightly cheekily get the count of the number of footprints in the query 
 * 
 */


function carbon_stamp_load($node)  {
		
	$query = db_query(
	"SELECT t.* , j.fid as attached_fid, n.title as attached_title " .
	"FROM {carbon_stamp} t " .
  "LEFT  JOIN {carbon_stamp_footprint} j ON t.nid = j.sid " .
  "LEFT  JOIN {node}                   n ON j.fid = n.nid " . // to get footprint title
	"WHERE t.nid = %d " .
	" ", $node->nid);
  $first_record = null ; 
  global $fid_map ; $fid_map  = array() ;  // having to hold this array because it is being wiped. 
  while ($record = db_fetch_object($query))
  {
  	if (empty($first_record))
    {
    	$first_record = $record ; 
      $first_record->fid_map = array();
    }
    if (!empty($record->attached_fid))
      $first_record->fid_map[$record->attached_fid] = $record->attached_title ; 
    $fid_map = $first_record->fid_map ; 
  }
  if (empty($first_record))
    drupal_set_message("Failed to load carbon_stamp where nid=", $node->nid);

	return $first_record;
}


/**
 * Implementation of hook_insert, which saves carbon-specific information
 * into the carbon table
 * @param node object
 */
 
function carbon_stamp_insert(&$node) 
{

  db_query("INSERT INTO {carbon_stamp} ".
	"(code, scope, reading, adjustment, protection, " .
	" exclude, startdate, enddate, nid) ".
	"VALUES ('%s', %d, '%s', '%s', %d, %d, %d, %d, %d)", 
	$node->code, $node->scope, $node->reading, $node->adjustment, 
	$node->protection, $node->exclude, $node->startdate, $node->enddate,
	$node->nid);   
  _update_footprint_join($node);   
}



/**
 * Implementation of hook_update, which saves updated todo-specific 
 * information into the todo table
 * @param node object
 */
function carbon_stamp_update(&$node) 
{

  db_query("UPDATE {carbon_stamp} set code='%s',scope=%d,reading='%s',adjustment='%s',protection=%d," .
    		" exclude=%d, startdate=%d, enddate=%d " .
    		" WHERE nid=%d", 
	$node->code, $node->scope, $node->reading, $node->adjustment,
	$node->protection, $node->exclude, $node->startdate, $node->enddate,
 	$node->nid);
  _update_footprint_join($node);   
}

/**
 * the use may choose to insert and delete the stamp from different 
 * footprints. carbon_stamp_submit() works out which tables need 
 * updating. This function inserts/removes the row.
 */


function _update_footprint_join($node)   
{	
  foreach ($node->fid_insert as $fid) 
  {
    db_query("INSERT INTO {carbon_stamp_footprint} (sid,fid) VALUES (%d, %d)", $node->nid, $fid);
  }

  foreach ($node->fid_delete as $fid)    
  {
    db_query("DELETE FROM {carbon_stamp_footprint} WHERE sid=%d AND fid=%d", $node->nid, $fid);
  }
}


/**
 * Implementation of hook_delete()
 */
function carbon_stamp_delete($node) {
	db_query("DELETE FROM {carbon_stamp_footprint} WHERE sid = %d", $node->nid);   
	db_query("DELETE FROM {carbon_stamp}           WHERE nid = %d", $node->nid);
}


/**
 * Implementation of hook_view, add our node specific information
 * @param node object to display
 * @param boolean is this a teaser or full node?
 * @param boolean is this displaying on its own page
 * 
 */
function carbon_stamp_view(&$node, $teaser = FALSE, $page = FALSE) 
{
//	if (!user_access('view carbon footprints')) {
//		return true;
//	}

  $node = node_prepare($node, $teaser);
  $carbon_footprint_info = theme('carbon_stamp_table_view', $node);  
  $node->body .= $carbon_footprint_info;
  $node->teaser .= $carbon_footprint_info;
}



function theme_carbon_stamp_table_view($node)
{
  $header = array(
    array("data" => t("date")),
    array("data" => t("code")),
    array("data" => t("reading")),
    array("data" => t("protected"))
  );
  global $fid_map ; 
  $attached_footprint = '' ;
  foreach ($fid_map as $fid => $title)
  {
  	$attached_footprint .= l($title. " ", "node/".$fid);
  }

	// content variable that will be returned for display

	$rows[] = array("title", $node->title);
  $rows[] = array("model", $node->model);
	$rows[] = array("code", $node->code. "  ");
	$rows[] = array("reading",$node->reading );
	$rows[] = array("adjustment",$node->adjustment);
	$rows[] = array("scope", _get_scope_description($node->scope));
	$rows[] = array("start date", empty($node->startdate) ? "" : format_date($node->startdate ));

	$rows[] = array("end date", format_date($node->enddate ));
//$rows[] = array("exclude", $node->exclude);
	$rows[] = array("created", format_date($node->created ));
	$rows[] = array("changed", format_date($node->changed ));
	$rows[] = array("published", $node->status);
	$rows[] = array("footprints attached", $attached_footprint);
	
	$output = theme("table", null, $rows);

  return $output;
}

/**
 * Implementation of hook_view, add our node specific information
 * @param node object to display
 * @param boolean is this a teaser or full node?
 * @param boolean is this displaying on its own page
 * 
 */
function carbon_stamp_edit(&$node, $teaser = FALSE, $page = FALSE) 
{
//  if (!user_access('view carbon footprints')) {
//      return true;
//  }

  $node = node_prepare($node, $teaser);
  $carbon_footprint_info = theme('carbon_stamp_table_view', $node);  
  $node->body .= $carbon_footprint_info;
  $node->teaser .= $carbon_footprint_info;
}


/**
 * Theme carbon table edit form  (this is not linked to menus yet)
 */
function theme_carbon_stamp_table_edit($form) {
  // Overview table:
  $header = array(NULL, t('Title'), t('Scope'), t('Code'), t('Reading'));
  $output = '' ; 
  $output .= form_render($form['account']);

    
  for ($g = 0 ;  ; $g++)
  {
      $rows = array(); 
      $stamp_group_form = "form.".$g ; 
      if (!isset($form[$stamp_group_form])) 
        break ; 
        
      foreach (element_children($form[$stamp_group_form]) as $stamp_form) {
        $row = array();
        $subform = $form[$stamp_group_form][$stamp_form];
        $row[] = _form_render($subform[$stamp_form.".title"]);
        $row[] = _form_render($subform[$stamp_form.".code_index"]);
        $row[] = _form_render($subform[$stamp_form.".reading"]);
        $row[] = _form_render($subform[$stamp_form.".startdate_s"]);
        $row[] = _form_render($subform[$stamp_form.".enddate_s"]);
        $rows[] = $row ;   
      }                      
      $output .= theme('table', $header, $rows);

  }
  $output .= form_render($form);
  if ($form['pager']['#value']) {
    $output .= form_render($form['pager']);
  }   
  return $output;
}


/**
 * remove all the text fields surround the box so it takes
 * up less space around 
 */

function _form_render($form)
{
    unset($form["#title"]);
    unset($form["#description"]);
    return form_render($form);	
}




?>